/*
 * CrossSectionDisplay.java
 *
 * Created on March 12, 2007, 11:06 AM
 *
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Point;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import javax.swing.JPanel;

/**
 * This object handles the display
 * @author cwarren
 */
public class CrossSectionDisplay  extends JPanel
{
    private CrossSection xSect;
    
    private boolean addingPebble = false;
    private boolean settingLongAxis = false;
    private boolean settingShortAxis = false;
    
    private int newPebLongLen;
    private int newPebShortLen;
    private double newPebAngle;
    private Point newPebCenter;

    private Point newPebEnd;

    private Color newPebColor = Color.BLUE;
    
    private Pebble newPeb;
    
    /** Creates a new instance of CrossSectionDisplay */
    public CrossSectionDisplay ()
    {
        this (new CrossSection ());
    }
    
    /** Creates a new instance of CrossSectionDisplay */
    public CrossSectionDisplay (CrossSection xSect)
    {
        this.setXSect (xSect);
    }
    
    public void paintComponent (Graphics g)
    {
        super.paintComponent (g);
        
        g.setColor (Color.WHITE);
        g.fillRect (0,0, this.getWidth (), this.getHeight ());
        
        Graphics2D g2 = (Graphics2D) g;
        
        AffineTransform deformer = new AffineTransform ();
        AffineTransform chartDeformer = new AffineTransform (); // no scales
        
        // tracks the original center so the image can be positioned appropriately
        Point crosshair = new Point (xSect.DIMX / 2, xSect.DIMX / 2);
        Point transCrosshairShearX = new Point ();
        Point transCrosshairTransX = new Point ();
        Point transCrosshairShearY = new Point ();
        Point transCrosshairTransY = new Point ();
        
        int xHalf = (xSect.DIMX / 2);
        int yHalf = (xSect.DIMY / 2);
        
        int recenterX,recenterY;
        
        // dragging transform
        if ((this.getXSect ().getOffsetX () != 0) || (this.getXSect ().getOffsetY () != 0))
        {
            deformer.concatenate (AffineTransform.getTranslateInstance (this.getXSect ().getOffsetX (),this.getXSect ().getOffsetY ()));
            chartDeformer.concatenate (AffineTransform.getTranslateInstance (this.getXSect ().getOffsetX (),this.getXSect ().getOffsetY ()));
        }
        // rotate transform
        if (this.getXSect ().getDisplayAngle () != 0)
        {
            deformer.concatenate (AffineTransform.getRotateInstance (this.getXSect ().getDisplayAngle (),xHalf,yHalf));
            chartDeformer.concatenate (AffineTransform.getRotateInstance (this.getXSect ().getDisplayAngle (),xHalf,yHalf));
        }
        
        // pure shear transform (i.e. X and Y scale)
        if ((this.getXSect ().getScaleX () != 1.0) || (this.getXSect ().getScaleY () != 1.0))
        {
            AffineTransform stretcher = AffineTransform.getScaleInstance (this.getXSect ().getScaleX (),this.getXSect ().getScaleY ());
            Point stretchCrosshair = new Point ();
            stretcher.transform ((Point2D)(new Point (xHalf,yHalf)),(Point2D)stretchCrosshair);
            deformer.concatenate (stretcher);
            chartDeformer.concatenate (stretcher);
            recenterX = (int)((double)(xHalf - stretchCrosshair.x)*(1.0/this.getXSect ().getScaleX ()));
            recenterY = (int)((double)(yHalf - stretchCrosshair.y)*(1.0/this.getXSect ().getScaleY ()));
            deformer.concatenate (AffineTransform.getTranslateInstance (recenterX,recenterY));
            chartDeformer.concatenate (AffineTransform.getTranslateInstance (recenterX,recenterY));
        }
        
        // magnification transform
        if (this.getXSect ().getMagnifierScale () != 1.0)
        {
            AffineTransform magnifier = AffineTransform.getScaleInstance (this.getXSect ().getMagnifierScale (),this.getXSect ().getMagnifierScale ());
            Point magnifyCrosshair = new Point ();
            magnifier.transform ((Point2D)(new Point (xHalf,yHalf)),(Point2D)magnifyCrosshair);
            deformer.concatenate (magnifier);
            recenterX = (int)((double)(xHalf - magnifyCrosshair.x)*(1.0/this.getXSect ().getMagnifierScale ()));
            recenterY = (int)((double)(yHalf - magnifyCrosshair.y)*(1.0/this.getXSect ().getMagnifierScale ()));
            deformer.concatenate (AffineTransform.getTranslateInstance (recenterX,recenterY));
        }        
        
        // "simple" shear transform
        AffineTransform xShearer = AffineTransform.getShearInstance (getXSect ().getShearX (),0);
        xShearer.transform (crosshair,(Point2D)transCrosshairShearX);
        deformer.concatenate (xShearer);
        chartDeformer.concatenate (xShearer);
        
        recenterX = xHalf - transCrosshairShearX.x;
        AffineTransform xShifter = AffineTransform.getTranslateInstance (recenterX,0);
        xShifter.transform ((Point2D)transCrosshairShearX,(Point2D)transCrosshairTransX);
        deformer.concatenate (xShifter);
        chartDeformer.concatenate (xShifter);
        
        AffineTransform yShearer = AffineTransform.getShearInstance (0,getXSect ().getShearY ());
        yShearer.transform ((Point2D)transCrosshairTransX,(Point2D)transCrosshairShearY);
        deformer.concatenate (yShearer);
        chartDeformer.concatenate (yShearer);
        
        recenterY = yHalf - transCrosshairShearY.y;
        AffineTransform yShifter = AffineTransform.getTranslateInstance (0,recenterY);
        yShifter.transform ((Point2D)transCrosshairShearY,(Point2D)transCrosshairTransY);
        deformer.concatenate (yShifter);
        chartDeformer.concatenate (yShifter);
        
                
        this.getXSect ().transformMe (deformer, chartDeformer);
        
        g2.drawImage (getXSect ().getCrossSectionImage (),null,0,0);
        
        if (this.isAddingPebble () && (null != this.newPeb))
        {
            Color origColor = g2.getColor ();            

            g2.setColor (this.newPeb.getColor ());

            newPeb.showOn (g2);
            
            g2.setColor (origColor);
        }
        
        // debugging info
        /*
        g.setColor (Color.MAGENTA);
        g2.drawString ("initial: "+xHalf + " "+yHalf,10,10);
        g2.drawString ("magnified: "+magnifyCrosshair.x + " "+magnifyCrosshair.y,10,30);
        g2.drawString ("recenter: "+recenterX + " "+recenterY,10,50);
        g2.drawString ("magnifier: "+this.getXSect ().getMagnifierScale (),10,70);
        g2.drawString ("shear X: "+transCrosshairShearX.x + " "+transCrosshairShearX.y,10,30);
        g2.drawString ("recenter X: "+recenterX,150,30);
        g2.drawString ("trans X: "+transCrosshairTransX.x + " "+transCrosshairTransX.y,10,50);
        g2.drawString ("shear Y: "+transCrosshairShearY.x + " "+transCrosshairShearY.y,10,70);
        g2.drawString ("recenter Y: "+recenterY,300,70);
        g2.drawString ("trans Y: "+transCrosshairTransY.x + " "+transCrosshairTransY.y,10,90);
         */
    }
    
    public CrossSection getXSect ()
    {
        return xSect;
    }
    
    public void setXSect (CrossSection xSect)
    {
        this.xSect = xSect;
    }
    
    public boolean isAddingPebble ()
    {
        return addingPebble;
    }
    
    public void setAddingPebble (boolean addingPebble)
    {
        this.addingPebble = addingPebble;
    }
    

    public Pebble getNewPeb ()
    {
        return newPeb;
    }

    public void setNewPeb (Pebble newPeb)
    {
        this.newPeb = newPeb;
    }

    public boolean isSettingLongAxis ()
    {
        return settingLongAxis;
    }

    public void setSettingLongAxis (boolean settingLongAxis)
    {
        this.settingLongAxis = settingLongAxis;
    }

    public boolean isSettingShortAxis ()
    {
        return settingShortAxis;
    }

    public void setSettingShortAxis (boolean settingShortAxis)
    {
        this.settingShortAxis = settingShortAxis;
    }

    public int getNewPebLongLen ()
    {
        return newPebLongLen;
    }

    public void setNewPebLongLen (int newPebLongLen)
    {
        this.newPebLongLen = newPebLongLen;
    }

    public double getNewPebAngle ()
    {
        return newPebAngle;
    }

    public void setNewPebAngle (double newPebAngle)
    {
        this.newPebAngle = newPebAngle;
    }

    public Point getNewPebCenter ()
    {
        return newPebCenter;
    }

    public void setNewPebCenter (Point newPebCenter)
    {
        this.newPebCenter = newPebCenter;
    }

    public int getNewPebShortLen ()
    {
        return newPebShortLen;
    }

    public void setNewPebShortLen (int newPebShortLen)
    {
        this.newPebShortLen = newPebShortLen;
    }

    public Point getNewPebEnd ()
    {
        return newPebEnd;
    }

    public void setNewPebEnd (Point newPebEnd)
    {
        this.newPebEnd = newPebEnd;
    }

    public Color getNewPebColor ()
    {
        return newPebColor;
    }

    public void setNewPebColor (Color newPebColor)
    {
        this.newPebColor = newPebColor;
    }
    
}
