/*
 * Pebble.java
 *
 * Created on March 13, 2007, 9:32 AM
 *
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.AffineTransform;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.geom.PathIterator;
import java.util.ArrayList;

/**
 * This class represents a single pebble (ellipsoid) in a cross section of rock.
 * That is, an ellipse of a specific ratio, angle, color, and position (center, 
 * not bounding-box corner). 
 * It implements various inspection methods and also knows how to transform and 
 * display itself. It tracks various information about how it is to be displayed, 
 * as well as its selection state. It supports a wide range of constructors, and 
 * support de/serialization. Finally, it implements the Cloneable interface.
 * 
 * 2009/05/25 - aded a snapshot history. NOTE: snapshot history is NOT included 
 * in clone - it contains deformed clones of this!
 * 
 * @author cwarren
 * @see Cloneable
 */
public class Pebble implements Cloneable
{
    /**
     * information about the origninal, undeformed version of this pebble - the center
     */
    protected Point origCenter;
    /**
     * information about the origninal, undeformed version of this pebble - one 
     * of the key points of the short axis. The key points of the short axis are the 
     * points that lie on the edge of the ellipse which are farthest from the center of 
     * the ellipse.
     */
    protected Point origLaKeyPt1;
    /**
     * information about the origninal, undeformed version of this pebble - the other 
     * of the key points of the short axis. The key points of the short axis are the 
     * points that lie on the edge of the ellipse which are farthest from the center of 
     * the ellipse.
     */
    protected Point origLaKeyPt2;
    /**
     * information about the origninal, undeformed version of this pebble - one 
     * of the key points of the short axis. The key points of the short axis are the 
     * points that lie on the edge of the ellipse which are closest to the center of 
     * the ellipse.
     */
    protected Point origSaKeyPt1;
    // the original key points of the ellipse, the center and where the axes intersect the arc
    /**
     * information about the origninal, undeformed version of this pebble - the other 
     * of the key points of the short axis. The key points of the short axis are the 
     * points that lie on the edge of the ellipse which are closest to the center of 
     * the ellipse.
     */
    protected Point origSaKeyPt2;
    /**
     * information about the origninal, undeformed version of this pebble - the angle 
     * in radians which the long axis diverges from the horizontal (anchor is left end,
     * postive is counter-clockwise)
     */
    protected double origThetaRad;
    /**
     * information about the origninal, undeformed version of this pebble - the Shape 
     * (Ellipse2D.Double) object representing this pebble
     */
    protected Ellipse2D.Double baseEllipse;
    /**
     * information about the current version of this pebble - the center
     */
    protected Point center;
    /**
     * information about the current version of this pebble - one 
     * of the key points of the short axis. The key points of the short axis are the 
     * points that lie on the edge of the ellipse which are farthest from the center of 
     * the ellipse.
     */
    protected Point laKeyPt1;
    /**
     * information about the current version of this pebble - the other 
     * of the key points of the short axis. The key points of the short axis are the 
     * points that lie on the edge of the ellipse which are farthest from the center of 
     * the ellipse.
     */
    protected Point laKeyPt2;
    /**
     * information about the current version of this pebble - one 
     * of the key points of the short axis. The key points of the short axis are the 
     * points that lie on the edge of the ellipse which are closest to the center of 
     * the ellipse.
     */
    protected Point saKeyPt1;
    
    // the active key points of the ellipse, the center and where the axes intersect the arc
    /**
     * information about the current version of this pebble - the other 
     * of the key points of the short axis. The key points of the short axis are the 
     * points that lie on the edge of the ellipse which are closest to the center of 
     * the ellipse.
     */
    protected Point saKeyPt2;
    /**
     * information about the current version of this pebble - the angle 
     * in radians which the long axis diverges from the horizontal (anchor is left end,
     * postive is counter-clockwise)
     */
    protected double thetaRad;
    
    /**
     * the Color used when this pebble is displayed
     */
    protected Color pebbleColor;
    
    /**
     * The original, undeformed shape object for this pebble
     */
    protected Shape basePebbleShape;
    /**
     * The current shape object for this pebble
     */
    protected Shape pebbleShape;
    /**
     * An AffineTransform which may be used to create an undeformed copy of a shape object.
     */
    protected final static AffineTransform shapeCopier = AffineTransform.getTranslateInstance (0,0);

    /**
     * working variable - holds long axis info while doing calculations
     */
    protected Point longAxisKey;
    
    /**
     * working variable - holds short axis info while doing calculations
     */
    protected Point shortAxisKey;
    
    // flags that determine how the pebble is displayed
    /**
     * flag use when the pebble is displayed; if true then the axes of the pebble are 
     * drawn along with the outline
     */
    protected boolean showAxes = false;
    /**
     * flag used when the pebble is displayed; if true then the pebble is drawn solid 
     * rather than just outlined
     */
    protected boolean fillEllipse = false;
    
    /**
     * flag that tracks whether or not this pebble is selected. This affects the 
     * display of the pebble somewhat (selected pebbles are highlighted), and it's used
     * in various other processed (e.g. deleting pebbles)
     */
    protected boolean selected = false;
    
    /**
     * list of copies of this pebble at various times.
     */
    protected ArrayList snapshots = new ArrayList();
    
    /**
     * A constant indicating how close to the center of a pebble the user must click
     * to select that pebble. Also used to display the 'handle' at the center of a 
     * pebble.
     */
    public static int SELECTION_RADIUS = 6;
    
    /**
     * The Color used to highlight a selected pebble when it's displayed
     */
    public static Color SELECTION_COLOR = Color.LIGHT_GRAY;
    /**
     * The Color used to highlight the point representing a selected pebble in a plot
     */
    public static Color SELECTION_PLOT_COLOR = Color.DARK_GRAY;
    
    /**
     * minimum long axis when adding a new pebble
     */
    public static int MIN_LONG_AXIS = 20;
        
    //**************************************************************************
    // CONSTRUCTORS
    //**************************************************************************
    
    /** 
     * Creates a new instance of Pebble with trivial default (centered on (0,0),
     * long and short axes both 0 length, angle of 0 (flat), and colored black)
     */
    public Pebble ()
    {
        this (0,0,0,0,0.0, Color.BLACK);
    }
    
    /**
     * 
     * Creates a new instance of Pebble with the specified attributes and 
     * colored black - this constructor support the angle being specified in
     * radians as a double
     * @param x The x coordinate of the center of the pebble
     * @param y The y coordinate of the center of the pebble
     * @param longAxis The length of the long axis, in pixels
     * @param shortAxis the length of the short axis, in pixels
     * @param thetaRad the angle of the long axis away from horizontal, in fractional radians (positive is CCW)
     */
    public Pebble (int x, int y, int longAxis, int shortAxis, double thetaRad)
    {
        this (x,y,longAxis,shortAxis,thetaRad, Color.BLACK);
    }
    
    /** 
     * Creates a new instance of Pebble with the specified attributes and 
     * colored black - this constructor support the angle being specified in
     * degrees as an integer
     * @param x The x coordinate of the center of the pebble
     * @param y The y coordinate of the center of the pebble
     * @param longAxis The length of the long axis, in pixels
     * @param shortAxis the length of the short axis, in pixels
     * @param thetaDeg the angle of the long axis away from horizontal, in integer degrees (positive is CCW)
     */
    public Pebble (int x, int y, int longAxis, int shortAxis, int thetaDeg)
    {
        this (x,y,longAxis,shortAxis,Math.toRadians (thetaDeg), Color.BLACK);
    }
    
    /** 
     * Creates a new instance of Pebble with the specified attributes and - this
     * constructor support the angle being specified in degrees as an integer
     * @param x The x coordinate of the center of the pebble
     * @param y The y coordinate of the center of the pebble
     * @param longAxis The length of the long axis, in pixels
     * @param shortAxis the length of the short axis, in pixels
     * @param thetaDeg the angle of the long axis away from horizontal, in integer degrees (positive is CCW)
     * @param c the color of the pebble
     */
    public Pebble (int x, int y, int longAxis, int shortAxis, int thetaDeg, Color c)
    {
        this (x,y,longAxis,shortAxis,Math.toRadians (thetaDeg), c);
    }
    
    /** 
     * Creates a new instance of Pebble with the specified attributes and - this
     * constructor support the angle being specified in radians as a double
     * @param x The x coordinate of the center of the pebble
     * @param y The y coordinate of the center of the pebble
     * @param longAxis The length of the long axis, in pixels
     * @param shortAxis the length of the short axis, in pixels
     * @param thetaRad the angle of the long axis away from horizontal, in fractional radians (positive is CCW)
     * @param c the color of the pebble
     */
    public Pebble (int x, int y, int longAxis, int shortAxis, double thetaRad, Color c)
    {
        this.setOrigCenter (new Point (x, y));
        this.setOrigLaKeyPt1 (new Point (x-(longAxis/2), y));
        this.setOrigLaKeyPt2 (new Point (x+(longAxis/2), y));
        this.setOrigSaKeyPt1 (new Point (x, y-(shortAxis/2)));
        this.setOrigSaKeyPt2 (new Point (x, y+(shortAxis/2)));
        this.setOrigThetaRad (thetaRad);
        
        this.copyOrigKeysToActiveKeys ();
        
        longAxisKey = (Point) this.getOrigLaKeyPt1 ().clone ();
        shortAxisKey = (Point) this.getOrigSaKeyPt1 ().clone ();
        
        this.baseEllipse = new Ellipse2D.Double (x-(longAxis/2),y-(shortAxis/2),longAxis,shortAxis);
        AffineTransform rotateTheta = new AffineTransform ();
        rotateTheta.rotate (this.getOrigThetaRad (),this.getOrigCenter ().x,this.getOrigCenter ().y);
        this.basePebbleShape = rotateTheta.createTransformedShape (this.baseEllipse);
        
        this.setPebbleColor (c);
        this.transformMe (null);
    }
    
    /**
     * String based contructor. Takes a string of the form:
     * x=200,y=300,shortAxis=25,longAxis=75,thetaDeg=45,color=#ff0000
     * x, y, longAxis, shortAxis, and thetaDeg must be integers
     * thetaDeg may be replaced with thetaRad and a double precision number
     * color is a hexidecimal number representing a 24-bit color, coded rrggbb
     * This string is the format which is used to represent a pebble in a 
     * GeoShear save file.
     * @param s A string which contains the data necessary to create the pebble
     */    
    public Pebble (String s)
    {
        int sx = 5;
        int sy = 5;
        int sLongAxis = 10;
        int sShortAxis = 5;
        int sThetaDeg = 0;
        double sThetaRad = 0.0;
        this.setPebbleColor (Color.CYAN);
        
        
        String[] params = s.split (",");
        
        for (int i=0; i<params.length; i++)
        {
            String[] keyVal = params[i].split ("=");
            if (keyVal[0].equalsIgnoreCase ("x"))
            { sx = Integer.parseInt (keyVal[1]); }
            else if (keyVal[0].equalsIgnoreCase ("y"))
            { sy = Integer.parseInt (keyVal[1]); }
            else if (keyVal[0].equalsIgnoreCase ("longAxis"))
            { sLongAxis  = Integer.parseInt (keyVal[1]); }
            else if (keyVal[0].equalsIgnoreCase ("shortAxis"))
            { sShortAxis = Integer.parseInt (keyVal[1]); }
            else if (keyVal[0].equalsIgnoreCase ("thetaRad"))
            { sThetaRad = Double.parseDouble (keyVal[1]); }
            else if (keyVal[0].equalsIgnoreCase ("thetaDeg"))
            { sThetaDeg = Integer.parseInt (keyVal[1]); }
            else if (keyVal[0].equalsIgnoreCase ("colorHex"))
            { this.setPebbleColor (Color.decode (keyVal[1])); }
            else if (keyVal[0].equalsIgnoreCase ("colorInt"))
            { 
                String[] colorVals = keyVal[1].split (":");
                this.setPebbleColor (new Color(Integer.parseInt (colorVals[0]),Integer.parseInt (colorVals[1]),Integer.parseInt (colorVals[2]))); 
            }
        }
        
        this.setOrigCenter (new Point (sx, sy));
        this.setOrigLaKeyPt1 (new Point (sx-(sLongAxis/2), sy));
        this.setOrigLaKeyPt2 (new Point (sx+(sLongAxis/2), sy));
        this.setOrigSaKeyPt1 (new Point (sx, sy-(sShortAxis/2)));
        this.setOrigSaKeyPt2 (new Point (sx, sy+(sShortAxis/2)));
                
        if (sThetaRad != 0.0)
        { this.setOrigThetaRad (sThetaRad); }
        else if (sThetaDeg != 0)
        { this.setOrigThetaRad (Math.toRadians (sThetaDeg)); }
        
        longAxisKey = (Point) this.getOrigLaKeyPt1 ().clone ();
        shortAxisKey = (Point) this.getOrigSaKeyPt1 ().clone ();
        
        this.baseEllipse = new Ellipse2D.Double (sx-(sLongAxis/2),sy-(sShortAxis/2),sLongAxis,sShortAxis);
        AffineTransform rotateTheta = new AffineTransform ();
        rotateTheta.rotate (this.getOrigThetaRad (),this.getOrigCenter ().x,this.getOrigCenter ().y);
        this.basePebbleShape = rotateTheta.createTransformedShape (this.baseEllipse);
        
        this.copyOrigKeysToActiveKeys ();
        this.transformMe (null);
    }
    
    //**************************************************************************
    // INTERFACE METHODS
    //**************************************************************************
    
    /**
     * Implementation of the clone method.
     * @return A new pebble that has all the same attribute values as this one
     */
    @Override
    public Pebble clone()
    {
        return new Pebble(this.serialize ());
    }    
    
    //**************************************************************************
    // 'REAL' METHODS
    //**************************************************************************
    
    /**
     * produces a string representing the original, undeformed pebble, suitable for later reading 
     * into the string-based constructor or saving to a file - this is identical to the toString method
     * e.g.  x=200,y=300,shortAxis=25,longAxis=75,thetaDeg=45,colorHex=#ff0000
     * @return a string representing the original, undeformed pebble, suitable for later reading 
     * into the string-based constructor or saving to a file
     */
    public String serialize ()
    {
        String pebbleString = "";
        
        pebbleString += "x="+this.getOrigCenter ().x+",";
        pebbleString += "y="+this.getOrigCenter ().y+",";
        pebbleString += "shortAxis="+this.getOrigShortAxisLength ()+",";
        pebbleString += "longAxis="+this.getOrigLongAxisLength ()+",";
        pebbleString += "thetaRad="+this.getOrigThetaRad ()+",";
        pebbleString += "colorHex=#" + java.lang.Integer.toHexString (this.getPebbleColor ().getRGB ()).substring (2);
        
        return pebbleString;
    }
    
    /**
     * produces a string representing the deformed version of this pebble, 
     * suitable for later reading into the string-based constructor or saving to
     * a file
     * e.g.  x=200,y=300,shortAxis=25,longAxis=75,thetaDeg=45,colorHex=#ff0000
     * @return a string representing the deformed version of this pebble, 
     * suitable for later reading into the string-based constructor or saving to
     * a file
     */
    public String serializeDeformed ()
    {
        String pebbleString = "";
        
        pebbleString += "x="+this.getCenter ().x+",";
        pebbleString += "y="+this.getCenter ().y+",";
        pebbleString += "shortAxis="+this.getShortAxisLength ()+",";
        pebbleString += "longAxis="+this.getLongAxisLength ()+",";
        pebbleString += "thetaRad="+this.getThetaRad ()+",";
        pebbleString += "colorHex=#" + java.lang.Integer.toHexString (this.getPebbleColor ().getRGB ()).substring (2);
        
        return pebbleString;
    }

    /**
     * produces a string representing the original, undeformed pebble, suitable for later reading 
     * into the string-based constructor or saving to a file - this is identical to the serialize method
     * e.g.  x=200,y=300,shortAxis=25,longAxis=75,thetaDeg=45,colorHex=#ff0000
     * @return a string representing the original, undeformed pebble, suitable for later reading 
     * into the string-based constructor or saving to a file
     */
    public String toString ()
    {
        return this.serialize ();
    }
    
    /**
     * Resets the numerical data for this pebble to the original values (i.e. the 
     * values used to create the pebble). In effect, this undeforms the pebble as far 
     * as numerical analysis is concerned.
     */
    public void copyOrigKeysToActiveKeys ()
    {
        center = (Point) this.getOrigCenter ().clone ();
        laKeyPt1 = (Point) this.getOrigLaKeyPt1 ().clone ();
        laKeyPt2 = (Point) this.getOrigLaKeyPt2 ().clone ();
        saKeyPt1 = (Point) this.getOrigSaKeyPt1 ().clone ();
        saKeyPt2 = (Point) this.getOrigSaKeyPt2 ().clone ();
        thetaRad = this.getOrigThetaRad ();
    }
    
    public int getLongAxisLength ()
    {
        return (int)(this.laKeyPt1.distance (this.laKeyPt2));
    }
    
    public int getShortAxisLength ()
    {
        return (int)(this.saKeyPt1.distance (this.saKeyPt2));
    }
    
    public int getOrigLongAxisLength ()
    {
        return (int)(this.origLaKeyPt1.distance (this.origLaKeyPt2));
    }
    
    public int getOrigShortAxisLength ()
    {
        return (int)(this.origSaKeyPt1.distance (this.origSaKeyPt2));
    }
    
    /**
     * Gives the area of the pebble, in pixels, computed using the formula for the 
     * area of an ellipse (A = PI * short axis * long axis)
     * @return The area of this pebble (ellipse) in pixels
     */
    public double getOrigArea ()
    {
        return Math.PI * (this.getOrigLongAxisLength ()/2.0) * (this.getOrigShortAxisLength ()/2.0);
    }
    
    /**
     * returns the ratio of the length of the short axis to the length of the long axis
     * @return the ratio of the length of the short axis to the length of the long axis
     */
    public double getRatioShortToLong ()
    {
        return (this.saKeyPt1.distance (this.saKeyPt2))/(this.laKeyPt1.distance (this.laKeyPt2));
    }
    
    /**
     * returns the ratio of the length of the long axis to the length of the short axis
     * @return the ratio of the length of the long axis to the length of the short axis
     */
    public double getRatioLongToShort ()
    {
        return ((this.laKeyPt1.distance (this.laKeyPt2)/this.saKeyPt1.distance (this.saKeyPt2)));
    }
    
    public int getThetaDeg ()
    {
        return (int)(Math.toDegrees (this.getThetaRad ()));
    }
        
    /**
     * Deforms this pebble by applying the given transform. This modifies the shape of 
     * the pebble and it's numbers (center point, axes, ratios, angles, etc.) 
     * If t is null then just the basic pebble is set (i.e. a given ellipse 
     * rotated a given amount)
     * @param transformer The AffineTransform to use to deform this pebble
     */
    @SuppressWarnings("static-access")
    public void transformMe (AffineTransform transformer)
    {
        this.copyOrigKeysToActiveKeys ();
        
        if (transformer == null)
        {
            transformer = this.shapeCopier;
        }
        
        this.pebbleShape = transformer.createTransformedShape (this.basePebbleShape);
        transformer.transform (this.center, this.center);
        
        this.determineLongAndShortAxis ();
    }
    
    /**
     * Figures out the (best approximation of the) long and short axes of this pebble 
     * based on the shape of the pebble. It does this by cycling through the points 
     * that make up the edge of the ellipse and getting the distance from each to the 
     * center.
     */
    protected void determineLongAndShortAxis ()
    {
        // cycle through the points of the arc, finding the distance to the center
        // figure out what and where the longest distance is
        // NOTE: need to go all the way through the loop - trying to short cut
        // after finding a single max lead to a radically sub-optimal choice
        double maxDist = 0;
        double minDist = 100000;
        double[] pts = {0,0,0,0,0,0};
        
        PathIterator pitr = pebbleShape.getPathIterator (null,.0001);
        while (! pitr.isDone ())
        {
            // NOTE: this modifies the pts array
            pitr.currentSegment (pts);
            
            // only the first two elements matter (because of the type of path iterator)
            double dist = this.center.distance (pts[0],pts[1]);
            if (pts[0] == 0 && pts[1] == 0)
            {
                pitr.next ();
                continue;
            }
            
            if (dist > maxDist)
            {
                maxDist = dist;
                this.longAxisKey.x = (int)pts[0];
                this.longAxisKey.y = (int)pts[1];
            }
            if (dist < minDist)
            {
                minDist = dist;
                this.shortAxisKey.x = (int)pts[0];
                this.shortAxisKey.y = (int)pts[1];
            }
            
            pitr.next ();
        }
        
        // determine long axis
        
        // determine the long axis slope and set the theta
        double laSlope = (double)(this.center.y - this.longAxisKey.y)/(double)(this.center.x - this.longAxisKey.x);
        this.thetaRad = Math.atan (laSlope);
        
        // set the long axis from the theta and length
        this.laKeyPt1 = new Point ((int)(this.center.x + (maxDist * Math.cos (this.thetaRad))),
                (int)(this.center.y + (maxDist * Math.sin (this.thetaRad))));
        this.laKeyPt2 = new Point ((int)(this.center.x - (maxDist * Math.cos (this.thetaRad))),
                (int)(this.center.y - (maxDist * Math.sin (this.thetaRad))));
        
        // short axis slope is perpendicular to long axis slope
        double saTheta = Math.atan (-1.0 / laSlope);
                
        // set the short axis from the theta and length
        this.saKeyPt1 = new Point ((int)(this.center.x + (minDist * Math.cos (saTheta))),
                (int)(this.center.y + (minDist * Math.sin (saTheta))));
        this.saKeyPt2 = new Point ((int)(this.center.x - (minDist * Math.cos (saTheta))),
                (int)(this.center.y - (minDist * Math.sin (saTheta))));

    }
    
    /**
     * Display this pebble in the given graphics context
     * @param g2d the graphics on/in which this pebble is displayed
     */
    public void showOn (Graphics2D g2d)
    {
        Color origColor = g2d.getColor ();
        
        g2d.setColor (this.getPebbleColor ());
        
        if (this.isFillEllipse ())
        {
            g2d.fill (this.pebbleShape);
        }
        else
        {
            g2d.draw (this.pebbleShape);
        }
        
        if (this.isShowAxes ())
        {
            // change color if the pebble is filled, otherwise the axes won't appear
            if (this.isFillEllipse ())
            {
                g2d.setColor (this.getPebbleColor ().darker ());
                if ( this.getPebbleColor ().getRed () + this.getPebbleColor ().getGreen () + this.getPebbleColor ().getBlue ()  <= 150)
                {
                    g2d.setColor (Color.LIGHT_GRAY);
                }
            }
            g2d.drawLine (laKeyPt1.x,laKeyPt1.y,laKeyPt2.x,laKeyPt2.y);
            g2d.drawLine (saKeyPt1.x,saKeyPt1.y,saKeyPt2.x,saKeyPt2.y);
        }
        
        g2d.setColor (origColor);
    }
    
    /**
     * show selection highlighting for this pebble in the given graphics
     * @param g2d the graphics on/in which to show the selection highlighting for this pebble
     */
    public void showSelectionOn (Graphics2D g2d)
    {
        if (this.isSelected ())
        {
            Color origColor = g2d.getColor ();
            Stroke origStroke = g2d.getStroke ();
            g2d.setStroke (new BasicStroke (5.0f));
            g2d.setColor (Pebble.SELECTION_COLOR);
            g2d.draw (this.pebbleShape);
            g2d.setStroke (origStroke);
            g2d.setColor (origColor);
        }
    }
    
    /**
     * Show selection handle for this pebble in the given graphics. The selection 
     * handle of a pebble is a region around the center of the pebble which the user 
     * clicking in causes this pebble to be selected.
     * @param g2d the graphics on/in which to show the selection handle for this pebble
     */
    public void showHandleOn (Graphics2D g2d)
    {
        Color origColor = g2d.getColor ();
        g2d.setColor (Color.WHITE);
        if (this.isSelected ())
        {
            g2d.setColor (Color.GREEN);
        }
        g2d.fillOval (this.getCenter ().x-Pebble.SELECTION_RADIUS,this.getCenter ().y-Pebble.SELECTION_RADIUS,Pebble.SELECTION_RADIUS*2,Pebble.SELECTION_RADIUS*2);
        g2d.setColor (Color.BLACK);
        g2d.drawOval (this.getCenter ().x-Pebble.SELECTION_RADIUS,this.getCenter ().y-Pebble.SELECTION_RADIUS,Pebble.SELECTION_RADIUS*2,Pebble.SELECTION_RADIUS*2);
        g2d.setColor (origColor);
    }

    /**
     * Stores a copy of the pebble in it's current state into snapshots
     */
    public void takeSnapshot()
    {
        this.snapshots.add(new Pebble(this.serializeDeformed ()));
        //this.snapshots.add(this.clone());
    }

    /**
     * Empties out the snapshots list
     */
    public void clearSnapshots()
    {
        this.snapshots.clear();
    }
    
    //**************************************************************************
    // GET/SET METHODS
    //**************************************************************************
    
    public int getX ()
    {
        return this.center.x;
    }
    
    public int getY ()
    {
        return this.center.y;
    }
    
    public double getThetaRad ()
    {
        return this.thetaRad;
    }
    
    public Color getPebbleColor ()
    {
        return this.pebbleColor;
    }
    
    public Color getColor ()
    {
        return this.getPebbleColor ();
    }
    
    public void setPebbleColor (Color pebbleColor)
    {
        this.pebbleColor = pebbleColor;
    }
    
    
    public Point getCenter ()
    {
        return this.center;
    }
    
    public void setCenter (Point center)
    {
        this.center = center;
    }
    
    public double getOrigThetaRad ()
    {
        return this.origThetaRad;
    }
    
    public int getOrigThetaDeg ()
    {
        return (int)(Math.toDegrees (this.origThetaRad));
    }
    
    public void setOrigThetaRad (double origThetaRad)
    {
        this.origThetaRad = origThetaRad;
    }
    
    public Point getOrigCenter ()
    {
        return this.origCenter;
    }
    
    public void setOrigCenter (Point origCenter)
    {
        this.origCenter = origCenter;
    }
    
    public Point getOrigLaKeyPt1 ()
    {
        return this.origLaKeyPt1;
    }
    
    public void setOrigLaKeyPt1 (Point origLaKeyPt1)
    {
        this.origLaKeyPt1 = origLaKeyPt1;
    }
    
    public Point getOrigLaKeyPt2 ()
    {
        return this.origLaKeyPt2;
    }
    
    public void setOrigLaKeyPt2 (Point origLaKeyPt2)
    {
        this.origLaKeyPt2 = origLaKeyPt2;
    }
    
    public Point getOrigSaKeyPt1 ()
    {
        return this.origSaKeyPt1;
    }
    
    public void setOrigSaKeyPt1 (Point origSaKeyPt1)
    {
        this.origSaKeyPt1 = origSaKeyPt1;
    }
    
    public Point getOrigSaKeyPt2 ()
    {
        return this.origSaKeyPt2;
    }
    
    public void setOrigSaKeyPt2 (Point origSaKeyPt2)
    {
        this.origSaKeyPt2 = origSaKeyPt2;
    }
    
    public boolean isShowAxes ()
    {
        return this.showAxes;
    }
    
    public void setShowAxes (boolean showAxes)
    {
        this.showAxes = showAxes;
    }
    
    public boolean isFillEllipse ()
    {
        return this.fillEllipse;
    }
    
    public void setFillEllipse (boolean fillEllipse)
    {
        this.fillEllipse = fillEllipse;
    }
    
    public boolean isSelected ()
    {
        return selected;
    }
    
    public void setSelected (boolean selected)
    {
        this.selected = selected;
    }

    public ArrayList getSnapshots() {
        return snapshots;
    }

    public void setSnapshots(ArrayList snapshots) {
        this.snapshots = snapshots;
    }
}
