/*
 * Utils.java
 *
 * Created on June 5, 2007, 2:55 PM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.io.File;

/**
 * This is a utility class to hold a number of commonly used functions
 * @author cwarren
 */
public class Utils
{

    /**
     * max number of places to show after the decimal on displayed numbers
     */
    public static int DISPLAY_PRECISION = 2; 
    
    /**
     * A number used in the truncation process
     */
    private static double TRUNCING_NUMBER = Math.pow (10.0,(double)Utils.DISPLAY_PRECISION);
    
    /** Creates a new instance of Utils */
    public Utils ()
    {
    }
    
    /**
     * truncates a text version of a decimal to a given number of places
     * e.g. truncTextDecimal("1.23456",2) -> "1.23". NOTE: this function may behave 
     * strangely if passed a string that does not represent a number!
     * @param textDecimal a string that has the text version of a decimal number
     * @param places an integer indicating the desired number of places after the decimal place
     * @return a version of the string that has at most 'places' characters after the 
     * decimal point
     */
    public static String truncTextDecimal(String textDecimal, int places)
    {
        if (places > 0) 
        {
            places = places + 1;
        }
        if (textDecimal.indexOf (".") > 0)
        {
            int truncLength = textDecimal.length () - textDecimal.indexOf (".");
            if (truncLength > places)
            {
                truncLength = places;
            }
            textDecimal = textDecimal.substring (0,textDecimal.indexOf (".")+truncLength);
        }
        return textDecimal;
    }
    
    /**
     * Truncates a given double to a fixed number of decimal places. The exact number 
     * of places is conrolled by the DISPLAY_PRECISION value in the Pebble class.
     * @param num the number to be truncated
     * @return the original number truncated to Pebble.DISPLAY_PRECISION places
     * @see Pebble
     */
    public static double truncForDisplay (double num)
    {   
        // the .001 mess is to deal with stupid floating point math issues
        if (num > 0)
        {
            return ((double)((int)(num * Utils.TRUNCING_NUMBER + .001)))/Utils.TRUNCING_NUMBER;
        } else if (num < 0)
        {
            return ((double)((int)(num * Utils.TRUNCING_NUMBER - .001)))/Utils.TRUNCING_NUMBER;            
        } else
        {
            return ((double)((int)(num * Utils.TRUNCING_NUMBER + .001)))/Utils.TRUNCING_NUMBER;
        }
    }
    
    /**
     * Truncates a given double to a fixed number of decimal places. The exact number 
     * of places is conrolled by the DISPLAY_PRECISION value in the Pebble class.
     * @param num the number to be truncated
     * @return the original number truncated to Pebble.DISPLAY_PRECISION places
     * @see Pebble
     */
    public static String truncForDisplay (String numAsString)
    {        
        return Utils.truncTextDecimal (numAsString, Utils.DISPLAY_PRECISION);
    }
    
    /**
     * A function to return the last (rightmost) N characters of a string. E.g.
     * rightmost("abcde",3) -> "cde"
     * @param s The string
     * @param numChars the number of characters to return
     * @return the string that's the rightmost (last) numChars characters of the string s
     * E.g. rightmost("abcde",3) -> "cde"
     */
    public static String rightmost(String s, int numChars)
    {
        return s.substring (s.length ()-numChars,s.length ());
    }
    
    /**
     * get the extension of the given file - copied from java.sun.com example
     * @param f The File of which the extension is desired
     * @return the extension of the file, defined as the substring from the rightmost "." in 
     * the file name ot the end of the file name.
     */
    public static String getExtension(File f) 
    {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }
    
    /**
     * Checks whether the points specified are l.t.e. dist apart
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param dist
     * @return true if points are less than dist apart, false otherwise
     */
    public static boolean pointsWithin(int x1, int y1, int x2, int y2,int dist)
    {
        return (Math.pow(x2 - x1,2) + Math.pow(y2 - y1,2)) <= Math.pow(dist,2);
    }
    
    /**
     * Gets a lighter version of a color, but never lighter than 245,245,245
     * 
     * @param initColor
     * @param factor a number between 0 and 1
     * @return
     */
    public static Color getLighterColor(Color initColor, double factor)
    {
        int max = 230;
        int iRed = initColor.getRed();
        int iGreen = initColor.getGreen();
        int iBlue = initColor.getBlue();
        
        iRed += (int)((255-iRed) * factor);
        iGreen += (int)((255-iGreen) * factor);
        iBlue += (int)((255-iBlue) * factor);
               
        return new Color(iRed < max ? iRed : max, iGreen < max ? iGreen : max,iBlue < max ? iBlue : max);
    }
}
